# -*- coding: utf-8 -*-
from datetime import datetime, date
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User

from jumbo.db.models import DateModel, UserModel, StatusModel, OrderedModel
from organization.models import Contact
from jumbo.apps.school.models import ClassRoom

#######################################
# APPUNTAMENTO                        #
#######################################
PERIODICITY = (
    ('d', _('all days')),
    ('w', _('all weeks')),
    ('m', _('all months')),
    ('y', _('all years')),
)

class Appointment(DateModel, UserModel, OrderedModel):
    ## ForeignKey
    #location = models.ForeignKey()
    assigned_by = models.ForeignKey(
        User,
        blank = True, null = True,
        related_name = "appointments_assigned_by_me",
        verbose_name = _("Assigned by"),
    )
    classes = models.ManyToManyField(
        ClassRoom,
        blank = True, null = True,
        db_index = True,
        related_name = "class_appointments",
        verbose_name = _("Class Appointments"),
    )
    persons = models.ManyToManyField(
        Contact,
        blank = True, null = True,
        db_index = True,
        through = 'Membership',
        related_name = "personal_appointments",
        verbose_name = _("Personal Appointments"),
    )
    ## Campi Modello
    title = models.CharField(
        max_length = 100,
        verbose_name = _("Title"),
    )
    description = models.TextField(
        blank = True, null = True,
        verbose_name = _("Description"),
    )
    date_begin = models.DateTimeField(
        db_index = True,
        default = datetime.today,
        verbose_name = _("Begin Date Time"),
    )
    date_end = models.DateTimeField(
        db_index = True,
        default = datetime.today,
        verbose_name = _("End Date Time"),
    )
    periodicity = models.CharField(
        max_length = 1,
        blank = True, null = True,
        choices = PERIODICITY,
        db_index = True,
        verbose_name = _("Periodicity"),
    )

    class Meta:
        verbose_name = _("Appointment")
        verbose_name_plural = _("Appointments")

    def __unicode__(self):
        return unicode('%s: %s - %s') % (self.title, self.date_begin.__str__(), self.date_end.__str__())

class DayPeriodicity(DateModel, UserModel):
    ## ForeignKey
    appointment = models.ForeignKey(
        Appointment,
        db_index = True,
        related_name = "day_periodicities",
        verbose_name = _("Appointment"),
    )
    description = models.TextField(
        blank = True, null = True,
        verbose_name = _("Description"),
    )
    repeat_days = models.IntegerField(
        verbose_name = _("Repeat Every Days"),
    )
    date_begin = models.DateField (
        blank = True, null = True,
        db_index = True,
        default = date.today,
        verbose_name = _("Begin Date"),
    )
    date_end = models.DateField (
        blank = True, null = True,
        db_index = True,
        default = date.today,
        verbose_name = _("End Date"),
    )

    class Meta:
        verbose_name = _("Day Appointment")
        verbose_name_plural = _("Day Appointments")

    def __unicode__(self):
        return unicode('%s: %s - %s') % (self.appointment.title, self.appointment.date_begin.__str__(), self.appointment.date_end.__str__())

class WeekPeriodicity(DateModel, UserModel):
    ## ForeignKey
    appointment = models.ForeignKey(
        Appointment,
        db_index = True,
        related_name = "week_periodicities",
        verbose_name = _("Appointment"),
    )
    description = models.TextField(
        blank = True, null = True,
        verbose_name = _("Description"),
    )
    repeat_weeks = models.IntegerField(
        verbose_name = _("Repeat Every Weeks"),
    )
    date_begin = models.DateField (
        blank = True, null = True,
        db_index = True,
        default = date.today,
        verbose_name = _("Begin Date"),
    )
    date_end = models.DateField (
        blank = True, null = True,
        db_index = True,
        default = date.today,
        verbose_name = _("End Date"),
    )
    mon = models.BooleanField(
        verbose_name = _("Monday"),
    )
    tue = models.BooleanField(
        verbose_name = _("Tuesday"),
    )
    wed = models.BooleanField(
        verbose_name = _("Wednesday"),
    )
    thu = models.BooleanField(
        verbose_name = _("Thursday"),
    )
    fri = models.BooleanField(
        verbose_name = _("Friday"),
    )
    sat = models.BooleanField(
        verbose_name = _("Saturday"),
    )
    sun = models.BooleanField(
        verbose_name = _("Sunday"),
    )

    class Meta:
        verbose_name = _("Week Appointment")
        verbose_name_plural = _("Week Appointments")

    def __unicode__(self):
        return unicode('%s: %s - %s') % (self.appointment.title, self.appointment.date_begin.__str__(), self.appointment.date_end.__str__())

TYPE_DAY = (
    ("w", _("Week Day"),),
    ("m", _("Month Day"),),
)

class MonthPeriodicity(DateModel, UserModel):
    ## ForeignKey
    appointment = models.ForeignKey(
        Appointment,
        db_index = True,
        related_name = "month_periodicities",
        verbose_name = _("Appointment"),
    )
    description = models.TextField(
        blank = True, null = True,
        verbose_name = _("Description"),
    )
    repeat_months = models.IntegerField(
        verbose_name = _("Repeat Every Months"),
    )
    date_begin = models.DateField (
        blank = True, null = True,
        db_index = True,
        default = date.today,
        verbose_name = _("Begin Date"),
    )
    date_end = models.DateField (
        blank = True, null = True,
        db_index = True,
        verbose_name = _("End Date"),
    )
    type_day = models.CharField(
        choices = TYPE_DAY,
        max_length = 1,
        db_index = True,
        verbose_name = _("Week Day"),
    )

    class Meta:
        verbose_name = _("Month Appointment")
        verbose_name_plural = _("Month Appointments")

    def __unicode__(self):
        return unicode('%s: %s - %s') % (self.appointment.title, self.appointment.date_begin.__str__(), self.appointment.date_end.__str__())

class YearPeriodicity(DateModel, UserModel):
    ## ForeignKey
    appointment = models.ForeignKey(
        Appointment,
        db_index = True,
        related_name = "year_periodicities",
        verbose_name = _("Appointment"),
    )
    description = models.TextField(
        blank = True, null = True,
        verbose_name = _("Description"),
    )
    repeat_years = models.IntegerField(
        verbose_name = _("Repeat Every Years"),
    )
    date_begin = models.DateField (
        blank = True, null = True,
        db_index = True,
        default = date.today,
        verbose_name = _("Begin Date"),
    )
    date_end = models.DateField (
        blank = True, null = True,
        db_index = True,
        verbose_name = _("End Date"),
    )

    class Meta:
        verbose_name = _("Year Appointment")
        verbose_name_plural = _("Year Appointments")

    def __unicode__(self):
        return unicode('%s: %s - %s') % (self.appointment.title, self.appointment.date_begin.__str__(), self.appointment.date_end.__str__())

#######################################
# RUOLO                               #
#######################################
class Role(DateModel, UserModel, OrderedModel):
    ## Campi Modello
    name = models.CharField (
        max_length = 100,
        db_index = True,
        verbose_name = _('Name'),
    )

    class Meta:
        verbose_name = _("Role")
        verbose_name_plural = _("Roles")

    def __unicode__(self):
        return u"%s" % self.name

STATUS_MEM = (
    ('A', _('ASSENTE')),
    ('O', _('OCCUPATO')),
    ('P', _('PRESENTE')),
)

#######################################
# INVITO                              #
#######################################
class Membership(models.Model):
    ## ForeignKey
    person = models.ForeignKey(
        Contact,
        db_index = True,
    )
    date_app = models.DateTimeField(
        db_index = True,
        verbose_name = _("Date Appointment"),
    )
    appointment = models.ForeignKey(
        Appointment,
        db_index = True,
    )
    role = models.ForeignKey(
        Role,
        db_index = True,
    )
    ## Campi Modello
    status = models.CharField (
        max_length = 1,
        blank = True, null = True,
        db_index = True,
        choices = STATUS_MEM,
        verbose_name = _("Status"),
    )
    invite_reason = models.CharField(
        max_length = 100,
        verbose_name = _("Ivite Reason"),
    )

    class Meta:
        verbose_name = _("Member Ship")
        verbose_name_plural = _("Member Ships")

    def __unicode__(self):
        return u"%s: %s - %s" % (self.appointment.title, self.appointment.date_begin.__str__(), self.appointment.date_begin.__str__())
