# -*- coding: utf-8 -*-

from datetime import datetime

from django.db import models
from django.contrib.auth.models import User, Group, Permission
from django.utils.translation import ugettext_lazy as _
from django.conf import settings as ls
from django.db.models import Q

from jumbo.db.models import DateModel, UserModel, StatusModel, OrderedModel
from jumbo.apps.document import settings
from organization.models import Organization, Contact
from jumbo.apps.project.models import Project
from jumbo.middleware.thread_local import get_current_user

############################
# PROGETTO                 #
############################
class DocumentProject(Project):
    managers = models.ManyToManyField (
        User,
        db_index = True,
        related_name = 'managed_documentprojects',
        verbose_name = _("Managers")
    )
    staff = models.ManyToManyField (
        User,
        db_index = True,
        related_name = 'assigned_documentprojects',
        verbose_name = _("Staff")
    )

    class Meta:
        verbose_name = _("Project")
        verbose_name_plural = _("Projects")
        permissions = (
            ("can_view_document_projects"   , "Can view document projects"),
            ("list_all_document_projects"   , "Can list all document projects"),
        )
        ordering = ['name']

    def __unicode__(self):
        return unicode(self.name)

#############################
# CATEGORIA                 #
#############################
class Category(DateModel, UserModel, OrderedModel, StatusModel):
    ## Campi
    code = models.CharField(
        unique = True,
        max_length = 15,
        db_index = True,
        verbose_name = _('Code'),
    )
    name = models.CharField(
        unique = True,
        max_length = 100,
        db_index = True,
        verbose_name = _('Name'),
    )
    ## email utilizzata per il supporto Scan2Email
    email = models.EmailField(
        unique = True,
        verbose_name = _('Email'),
        help_text = _('Used for Scan2Email')
    )
    note = models.TextField(
        blank = True, null = True,
        verbose_name = _("Note"),
    )

    class Meta:
        verbose_name = _("Category")
        verbose_name_plural = _("Categories")

    def __unicode__(self):
        return u"%s" % self.name

    def save(self):
        if not self.email:
            name = self.name.replace(' ', '_').lower()
            self.email = "%s@%s" % (name, settings.DOCUMENT_DOMAIN_NAME)

        super(Category, self).save()

#############################
# DOCUMENTO                 #
#############################
class DocumentManager(models.Manager):
    def get_query_set(self):
        user = get_current_user()

        if user.has_perm("document.list_all_documents"):
            return super(DocumentManager, self).get_query_set()

        if user.has_perm("document.list_own_group_documents"):
            return super(DocumentManager, self).get_query_set().filter(
                Q(is_public=True) |
                (Q(is_public=False) & Q(notify_group__in=user.groups.all()))
            ).distinct()

        return super(DocumentManager, self).get_query_set().filter(
            Q(is_public=True) |
            (Q(is_public=False) & Q(assigned_to=user))
        )

class Document(DateModel, UserModel, OrderedModel):
    # Foreign Key
    projects = models.ManyToManyField(
        DocumentProject,
        null = True, blank = True,
        db_index = True,
        related_name = "documents",
        verbose_name = _("Projects"),
    )
    notify_group = models.ManyToManyField (
        Group,
        blank = True, null = True,
        db_index = True,
        related_name = "documents",
        verbose_name = _("notify_group"),
    )
    organization = models.ForeignKey(
        Organization,
        null = True, blank = True,
        db_index = True,
        related_name = "documents",
        verbose_name = _("Organization"),
    )
    person = models.ForeignKey(
        Contact,
        null = True, blank = True,
        related_name = "documents",
        verbose_name = _("Person"),
    )
    assigned_by = models.ForeignKey (
        User,
        blank = True, null = True,
        related_name = "documents_assigned_by_me",
        verbose_name = _("Assigned by"),
    )
    assigned_to = models.ForeignKey (
        User,
        blank = True, null = True,
        related_name = "documents_assigned_to_me",
        verbose_name = _("Assigned to"),
    )
    category = models.ForeignKey(
        Category,
        db_index = True,
        verbose_name = _("Category"),
    )
    # Campi Modello
    status = models.CharField(
        max_length = 2,
        choices = settings.STATUS_DOC,
        db_index = True,
        verbose_name = _("Status"),
        help_text = _("Help text")
    )
    date_doc = models.DateTimeField(
        null = True, blank = True,
        db_index = True,
        verbose_name = _("Document Date and Time"),
        ## campo obbligatorio quando in stato "COMPLETATO"
    )
    number = models.CharField(
        null = True, blank = True,
        max_length = 20,
        db_index = True,
        verbose_name = _("Document Number"),
        ## obbligatorio per raccomandate
    )
    working = models.BooleanField(
        default = False,
        db_index = True,
        verbose_name = _("Working")
    )
    reception_date = models.DateTimeField(
        db_index = True,
        verbose_name = _("Reception Date"),
    )
    protocol_number = models.IntegerField(
        null = True, blank = True,
        db_index = True,
        verbose_name = _("Protocol Number"),
    )
    title = models.CharField(
        max_length = 100,
        db_index = True,
        verbose_name = _("Title"),
    )
    keywords = models.CharField(
        null = True, blank = True,
        max_length = 200,
        db_index = True,
        verbose_name = _("Keywords"),
    )
    pages = models.PositiveSmallIntegerField(
        null = True, blank = True,
        max_length = 3,
        verbose_name = _("Pages Number"),
    )
    support = models.CharField(
        max_length = 1,
        choices = settings.SUPPORTO_DOC,
        db_index = True,
        verbose_name = _("Support"),
    )
    file = models.FileField (
        upload_to = settings.DOCUMENT_PATH_DOCUMENTS,
        verbose_name = _("File PDF"),
    )
    note = models.TextField (
        null = True, blank = True,
        verbose_name = _("Note"),
    )
    is_public = models.BooleanField(
        default = False
    )
    objects = DocumentManager()

    class Meta:
        verbose_name = _("Document")
        verbose_name_plural = _("Documents")
        permissions = (
            ("can_view_documents",                  "Can view documents"),
            ("list_all_documents",                  "Can list all documents"),
            ("list_own_group_documents",            "Can list only own group documents"),
            ("change_own_documents",                "Can change only own documents"),
            ("delete_own_documents",                "Can delete only own documents"),
            ("can_download_documents",              "Can download documents"),
        )

    def get_string_projects(self):
        if self.projects:
            projects = [p.name for p in self.projects.all()]
            return ', '.join(projects)
        return ''

    def save(self):
        if self.date_doc and not self.protocol_number:
            self.protocol_number = self.get_new_protocol_number()
        from jumbo.middleware.thread_local import get_current_user
        user = get_current_user()
        if user:
            self.utente = user

        super(Document, self).save()

    def is_busy(self):
        ## un documento è impegnato da altri quando 'user' è diverso dal 'current_user'
        ## e self.documento_in_lavorazione è True
        from jumbo.middleware.thread_local import get_current_user
        user = get_current_user()
        list_perms = user.get_all_permissions()

        ## controlla se è STAFF
        if user.is_staff:
            return False

        if self.last_modifier != user and self.working == True:
            return True

        return False

    def can_be_added(self):
        from jumbo.middleware.thread_local import get_current_user
        user = get_current_user()
        list_perms = user.get_all_permissions()

        ## controlla permessi di modifica "STAFF"
        if user.is_staff:
            return True

        ## controlla se inserimento da WEB
        if "document.can_insert_by_web" in list_perms:
            return True

        return False

    def can_be_delete(self):
        from jumbo.middleware.thread_local import get_current_user
        user = get_current_user()

        ## controlla permessi di delete "ALL" o "STAFF"
        if user.has_perm("document.delete_document"):
            return True

        ## controlla permessi di delete "OWN"
        if user.has_perm("document.delete_own_documents"):
            user_groups = user.groups.all()
            notify_groups = self.notify_group.all()

            if self.assigned_to and self.assigned_to.user == user:
                return True
            for g in notify_groups:
                if g in user_groups:
                    return True

        return False

    def can_be_modified(self):
        from jumbo.middleware.thread_local import get_current_user
        user = get_current_user()

        ## controlla se il documento non è già impegnato da un altro
        if self.is_busy():
            return False

        ## controlla permessi di modifica "ALL" o "STAFF"
        if user.has_perm("document.change_document"):
            return True

        ## controlla permessi di modifica "OWN"
        if user.has_perm("document.change_own_documents"):
            user_groups = user.groups.all()
            notify_groups = self.notify_group.all()

            if self.assigned_to and self.assigned_to.user == user:
                return True
            for g in notify_groups:
                if g in user_groups:
                    return True

        return False


    def get_new_protocol_number(self):
        last_doc = Document.objects.filter(
            category = self.category,
            date_doc__year = self.date_doc.year
        ).order_by('-protocol_number')[0:1]
        if last_doc:
            return last_doc[0].protocol_number + 1
        else:
            return 1

    def set_num_doc(self, number):
        self.number = number
        self.save()

    def __unicode__(self):
         return u"%s/%s %s" % (self.protocol_number, self.date_doc, self.category.name)
