from django.contrib.auth.decorators import login_required
from django.http import HttpResponse, HttpResponseRedirect, HttpResponseNotFound, HttpResponseForbidden
from django.utils.translation import ugettext_lazy as _
from django.views.generic import list_detail

from jumbo.views.generic import generic_views
from jumbo.shortcuts import render_to_response, render_to_string
from jumbo.apps.document.models import Document, Category
from jumbo.apps.document.forms import DocumentForm, DocumentSearchForm
from jumbo.apps.document import settings
from jumbo.apps.document.list import DocumentList


@login_required
def document_list(request):
    if not request.user.has_perm("document.can_view_documents"):
        return HttpResponseForbidden(render_to_string('jumbo/forbidden.html'))

    if request.GET:
        search_form = DocumentSearchForm(request.GET)
    else:
        search_form = DocumentSearchForm()

    qs = generic_views.get_query_set(DocumentSearch, search_form)

    data_list = DocumentList()

    return list_detail.object_list(
        request,
        qs,
        paginate_by=20,
        template_name='document/document_list.html',
        extra_context={'search_form':search_form, 'data_list':data_list},
    )

@login_required
def document_add(request):
    if not request.user.has_perm("document.add_document"):
        return HttpResponseForbidden(render_to_string('jumbo/forbidden.html'))

    if request.method == 'POST':
        form = DocumentForm(request.POST, request.FILES)
        if form.is_valid():
            form.save()
            return HttpResponseRedirect('/document/')
    else:
        form = DocumentForm()

    title_form = _('Add Document')
    return render_to_response('document/document_form.html', form=form, title_form=title_form)

@login_required
def document_edit(request, object_id):
    if not request.user.has_perm("document.change_document") and \
       not request.user.has_perm("document.change_own_documents"):
        return HttpResponseForbidden(render_to_string('jumbo/forbidden.html'))

    document = Document.objects.get(pk=object_id)
    if request.method == 'POST':
        form = DocumentForm(request.POST, request.FILES, instance=document)
        if form.is_valid():
            form.save()
            return HttpResponseRedirect('/document/')
    else:
        form = DocumentForm(instance=document)
    return render_to_response('document/document_form.html', form=form, title_form=_('Change Document'))

@login_required
def document_delete(request, object_id):
    return render_to_response("admin/pp")

@login_required
def document_lock(request):
    return render_to_response("admin/pp")

@login_required
def document_download(request, object_id):
    from django.views.static import serve
    if not request.user.has_perm("document.can_download_documents"):
        return HttpResponseForbidden(render_to_string('jumbo/forbidden.html'))

    try:
        r = Document.objects.get(id=object_id)
        if not r.file:
            raise Exception
        return serve(request, r.file.path, '/')
    except Exception, e:
        return HttpResponseNotFound(render_to_string('jumbo/generic_error.html', error=e))


@login_required
def document_get_search_data(request):
    data = generic_views.get_search_data(DocumentSearch, DocumentSearchForm)
    return HttpResponse(data, mimetype='text/javascript;')

class DocumentSearch(object):
    default_order_by = '-date_doc'
    model = Document

    def change_filter(self, filtro):
        if filtro.has_key('date_doc_gte__gte'):
            date_doc_gte = filtro['date_doc_gte__gte']
            del filtro['date_doc_gte__gte']
            filtro['date_doc__gte'] = date_doc_gte

        if filtro.has_key('date_doc_lte__lte'):
            date_doc_lte = filtro['date_doc_lte__lte']
            del filtro['date_doc_lte__lte']
            filtro['date_doc__lte'] = date_doc_lte

        return filtro
