from django import forms
from django.forms.util import ErrorList, ValidationError, flatatt
from django.utils.hashcompat import md5_constructor
from django.utils.translation import ugettext as _
from django.utils.encoding import force_unicode
from django.utils.safestring import mark_safe
from django.utils.datastructures import MultiValueDict
from django.forms.fields import Field, UploadedFile, EMPTY_VALUES

from jumbo.forms.widgets import MultiFileInput


class HiddenInput(forms.Widget):
    is_hidden = True
    def render(self, name, value, attrs=None):
        if value is None:
            value = ''
        final_attrs = self.build_attrs(attrs, type='text', name=name)
        if value != '':
            # Only add the 'value' attribute if a value is non-empty.
            final_attrs['value'] = force_unicode(value)
        return mark_safe(u'''<div style="display: none;"><input%s /></div>''' % flatatt(final_attrs))

class NoSpamForm(forms.Form):
    def __init__(self, request, *args, **kws):
        super(NoSpamForm, self).__init__(*args, **kws)
        self.request = request
        self.key = str(request.session.session_key)
        self.value = md5_constructor(self.key).hexdigest()
        self.fields[self.key] = forms.CharField(required=False, widget=HiddenInput(), initial=self.value)
    def clean(self):
        if not (self.key in self.cleaned_data and self.cleaned_data[self.key] == self.value):
            raise forms.ValidationError(_(u'There was an unknown error submitting your request'))
        return self.cleaned_data

class MultiFileField(forms.Field):
    """
    A field allowing users to upload multiple files at once.
    """
    widget = MultiFileInput
    count = 1
    
    def __init__(self, count=1, strict=False, *args, **kwargs):
        """
        strict is whether the number of files uploaded must equal count
        """
        self.count = count
        self.strict = strict
        super(MultiFileField, self).__init__(*args, **kwargs)

    def widget_attrs(self, widget):
        """
        Adds the count to the MultiFileInput widget.
        """
        if isinstance(widget, MultiFileInput):
            return {'count':self.count}
        return {}

    def clean(self, data):
        """
        Cleans the data and makes sure that all the files had some content.
        Also checks whether a file was required.
        """
        super(MultiFileField, self).clean(data)

        if not self.required and data in EMPTY_VALUES:
            return None
        try:
            f = data
        except TypeError:
            raise ValidationError(_(u"No file was submitted. Check the encoding type on the form."))
        except KeyError:
            raise ValidationError(_(u"No file was submitted."))

        for a_file in f:
            if not a_file.size:
                raise ValidationError(_(u"The submitted file is empty."))
        
        if self.strict and len(f) != self.count:
            raise ValidationError(_(u"An incorrect number of files were uploaded."))

        return f

class FixedMultiFileInput(MultiFileInput):
    """
    A MultiFileInput widget that doesn't print the javascript code to allow
    the user to add more file input boxes.
    """
    def link(self, name, value, count, attrs=None):
        return u''
    
    def js(self, name, value, count, attrs=None):
        return u''
